// ---------------------------------------------------------------------------
//	M88 - PC-8801 Emulator
//	Copyright (C) cisc 1997, 1999.
// ---------------------------------------------------------------------------
//	$Id: sound.h,v 1.8 1999/06/19 14:06:24 cisc Exp $

#ifndef PC88_SOUND_H
#define PC88_SOUND_H

#include "device.h"
#include "opna.h"
#include "psgb.h"

// ---------------------------------------------------------------------------

#define SOUND_55K	55555	// Special

class PC88;
class Scheduler;

namespace PC8801
{
class Sound;
class Config;

// ---------------------------------------------------------------------------
//	88 p OPN Unit
//
class YM2608 : public Device
{
public:
	enum IDFunc
	{
		reset=0, setindex0, setindex1, writedata0, writedata1, setintrmask,
		readstatus=0, readstatusex, readdata0, readdata1,
	};

public:
	YM2608(const ID& id, int imaskbit);
	~YM2608();
	bool Init(Bus* bus, int intrport, int io, Scheduler* s, uint c, uint r);
	void SetSound(Sound* s) { sound = s; }
	void TimeEvent(uint=0);
	
	bool SetRate(uint c, uint r);
	void Mix(FM::Sample* buffer, int nsamples, int b1=0, int b2=0, int b3=0);
	void SetVolume(const Config* config);

	void SetIntrMask(uint, uint intrmask);
	void Enable(bool en) { enable = en; }
	
	void Reset		(uint=0, uint=0);
	void SetIndex0	(uint, uint data);
	void SetIndex1	(uint, uint data);
	void WriteData0	(uint, uint data);
	void WriteData1	(uint, uint data);
	uint ReadData0	(uint);
	uint ReadData1	(uint);
	uint ReadStatus	(uint);
	uint ReadStatusEx(uint);
	
	void SetOPNMode(bool _opna);
	const uint8* GetRegs() { return regs; }

	const Descriptor* GetDesc() const { return &descriptor; }

private:
	void UpdateTimer();

private:
	class OPNUnit : public FM::OPNA
	{
	public:
		OPNUnit() : bus(0) {}
		~OPNUnit() {}
		void Intr(bool f);
		void SetIntr(Bus* b, int p) { bus = b, pintr = p; }
		void SetIntrMask(bool e);

	private:
		Bus* bus;
		int pintr;
		bool intrenabled;
		bool intrpending;
	};

	OPNUnit opn;
	PSGB psg;
	Bus* bus;
	Scheduler* scheduler;
	Sound* sound;
	int32 clock;
	int32 nextcount;
	int imaskbit;
	int srate;
	int prevtime;
	int portio;

	uint index0;
	uint index1;
	uint data1;
	
	bool opnamode;
	bool newopnamode;
	bool enable;
	uint8 regs[0x200];

	static const Descriptor descriptor;
	static const InFuncPtr  indef[];
	static const OutFuncPtr outdef[];
};

// ---------------------------------------------------------------------------
//
//
class Sound : public Device
{
public:
	enum IDFunc
	{
		reset=0, rtc, out40
	};

public:
	Sound(const ID& id, const ID& idopn1, const ID& idopn2);
	~Sound();
	
	bool Init(PC88* pc, uint rate, uint32 clock, int bufsize);
	void Cleanup();	
	const Descriptor* GetDesc() const { return &descriptor; }
	
	void ApplyConfig(const Config* config);
	virtual void SetVolume(const Config* config);
	bool SetRate(uint rate, uint32 clock, int bufsize);
	void Fill(void* dest, int bytes);
	
	void Reset(uint=0, uint=0);
	void RTC(uint=0, uint=0);
	void Out40(uint, uint data);
	void UpdateBeepState();
	bool IsBufferEmpty();

	void OPNEvent();
	YM2608* GetOPN1() { return &opn1; }
	YM2608* GetOPN2() { return &opn2; }

protected:
	virtual void Store(int);
	
	int bwpos;
	int uscount;
	uint samplingrate;		// TvO[g
	uint srate50;			// samplingrate / 50

private:
	void Mix(void* dest, int bytes);

	PC88* pc;
	int32* mixingbuf;
	
	uint8* buffer;			// Oobt@֌W
	int buffersize;
	int read;
	int write;

	uint32 prevtime;
	
	uint32 cfgflg;

	bool enabled;
	bool opn2enabled;

	// BEEP ֌W
	uint port40;
	uint beepmask;
	uint pswitches;
	uint bswitches;
	uint bvalue;
	uint32 pcount[8], pindex, plasttime, pstep;
	uint32 bcount[8], bindex, blasttime, bstep;
	
protected:
	YM2608 opn1;
	YM2608 opn2;
	
	static const Descriptor descriptor;
//	static const InFuncPtr  indef[];
	static const OutFuncPtr outdef[];
};

}

#endif // PC88_SOUND_H
