// ---------------------------------------------------------------------------
//	M88 - PC-8801 Emulator.
//	Copyright (C) cisc 1997, 1999.
// ---------------------------------------------------------------------------
//	$Id: wincore.h,v 1.13 1999/06/20 05:24:03 cisc Exp $

#if !defined(win32_wincore_h)
#define win32_wincore_h

// ---------------------------------------------------------------------------

#include "types.h"
#include "CritSect.h"
#include "pc88/pc88.h"
#include "pc88/config.h"
#include "calender.h"
#include "winsound.h"
#include "winjoy.h"
#include "winmouse.h"

namespace PC8801
{
class WinKeyIF;
}

// ---------------------------------------------------------------------------

class WinCore
{
public:
	WinCore();
	~WinCore();
	bool Init(WinUI* ui, HWND hwnd, Draw* draw, PC8801::WinKeyIF* keyb); 
	bool Cleanup();

	void Reset();
	void ApplyConfig(PC8801::Config* config);
	void SetVolume(PC8801::Config* config);
	
	long GetExecCount();
	void SetTimeMode(bool useqpc);
	void Wait(bool dowait);
	void SetGUIFlag(bool flag);
	void ActivateMouse(bool flag);

	bool Mount(uint drive, const char* diskname, bool readonly, int index, bool create);
	bool Unmount(uint drive);
	const char* GetImageTitle(uint dr, uint index);
	uint GetNumDisks(uint dr);
	int GetCurrentDisk(uint dr); 
	bool AddDisk(uint dr, const char* title, uint type);
	bool FormatDisk(uint dr);
	bool IsImageOpen(const char* filename);

	const uint8* GetOPNReg() { return sound.GetOPN1()->GetRegs(); }
	PC88* GetPC88() { return &pc88; }

private:
	bool ConnectDevices(PC8801::WinKeyIF* keyb);
	void Execute(long clock, long time);
	bool EnablePad(bool enable);
	bool EnableMouse(bool enable);

	uint32 GetTime();
	
	uint ThreadMain();
	static uint __stdcall ThreadEntry(LPVOID arg);
	
	CriticalSection cs_cpu;
	WinUI* ui;

	HANDLE hthread;
	uint idthread;

	int clock;
	int execcount;
	uint32 precision;
	
	volatile bool shouldterminate;
	volatile bool execute;
	bool padenable;
	bool mouseenable;
	int guicount;

	PC88 pc88;
	PC8801::Calender caln;
	PC8801::WinSound sound;
	PC8801::WinJoyPad pad;
	PC8801::WinMouse mouse;
};

// ---------------------------------------------------------------------------
//	DiskManager ֌W

inline bool WinCore::Mount
(uint dr, const char* name, bool ro, int index, bool create)
{
	return pc88.GetDiskManager()->Mount(dr, name, ro, index, create);
}

inline bool WinCore::Unmount(uint dr)
{
	return pc88.GetDiskManager()->Unmount(dr);
}

inline const char* WinCore::GetImageTitle(uint dr, uint index)
{
	return pc88.GetDiskManager()->GetImageTitle(dr, index);
}

inline uint WinCore::GetNumDisks(uint dr)
{
	return pc88.GetDiskManager()->GetNumDisks(dr);
}

inline int WinCore::GetCurrentDisk(uint dr)
{
	return pc88.GetDiskManager()->GetCurrentDisk(dr);
}

inline bool WinCore::AddDisk(uint dr, const char* title, uint type)
{
	return pc88.GetDiskManager()->AddDisk(dr, title, type);
}

inline bool WinCore::FormatDisk(uint dr)
{
	return pc88.GetDiskManager()->FormatDisk(dr);
}

inline bool WinCore::IsImageOpen(const char* name)
{
	return pc88.GetDiskManager()->IsImageOpen(name);
}

inline void WinCore::SetVolume(PC8801::Config* config)
{
	sound.SetVolume(config);
}


#endif // !defined(win32_wincore_h)
